@@ -102,12 +102,14 @@ def convert_row(metadata, ws_data):
 def is_single_multiple_structure(column):
     column_type = column['type']
     if column_type in ('single-select', 'multiple-select'):
-        options = column.get('data', {}).get('options', [])
+        column_data = column.get('data', {})
+        options = column_data and column_data.get('options', []) or []
         return True, options
     if column_type in ('link', 'link-formula'):
         array_type = column.get('data', {}).get('array_type')
         if array_type in ('single-select', 'multiple-select'):
-            options = column.get('data', {}).get('array_data', {}).get('options', [])
+            array_data = column.get('data', {}).get('array_data', {})
+            options = array_data and array_data.get('options', []) or []
             return True, options
     return False, []
 
